 /**********************************************************************
 *
 *
 *		    ROVER object execution engine
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: HTCache.h
 *
 * Description: Persistent cache management
 *
 * Last Modified:  8-Feb-95 (adj)
 *
 * Global Functions:
 *    PUBLIC HTStream *HTAppRun(HTRequest *request,
 *				void *param,
 *				HTFormat input_format,
 *			        HTFormat output_format,
 *				HTStream *output_stream)
 *        Converter for application/x-application MIME stream. Downloads
 *        application binary code, creates pipes for IPC, and forks off
 *        a separate process.
 *
 * Global Variables: PUBLIC CONST HTStreamClass HTApplication
 *
 ***********************************************************************
 *   Copyright 1995
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#ifndef HTCACHE_H
#define HTCACHE_H

#define ROVER_CACHE_VERSION 3
#define ROVER_CACHE_DIRECTORY 	"rover-cache"
#define ROVER_CACHE_INFO 	"rover-cache-info"

extern int RoverCacheMemLimit;

extern RoverErr RoverCacheFlush(char *url, int session);
extern RoverErr RoverCacheInit();
extern RoverErr RoverCacheSave();
extern RoverErr RoverCacheInsertData(char *url, int session,
				     RoverObjType type, RoverCacheable cache,
				     char *data, unsigned long size,
				     time_t objTime, time_t delay,
				     time_t last, char *dv,
				     unsigned long dvsize);
extern RoverErr RoverCacheRef(char *url, int session, time_t last);
extern RoverErr RoverCacheUpdateName(char *url, int session, char *name);
extern RoverErr RoverCacheUpdate(char *url, int session, RoverObjType type,
				 RoverCacheable cache, char *name,
				 time_t objTime, time_t delay, time_t last,
				 int size, char *data, char *dv,
				 unsigned long dvsize);
extern RoverErr RoverCacheUpdateData(char *url, int session, char *data,
				     int size, char *dv,
				     unsigned long dvsize);
extern RoverErr RoverCacheUpdateDependV(char *url, int session, char *dv,
					unsigned long dvsize);
extern char *RoverCacheSearchName(char *url, int session, RoverCacheable *cache,
				  RoverObjType *type, char **dv,
				  unsigned long *dvsize);
extern char *RoverCacheSearchData(char *url, int session, int *size);
extern RoverErr RoverCacheRequest(char *url, int session,
				  RoverObjType type);
extern char *RoverCacheSearchByType(RoverObjType type, int *size,
				    time_t *objTime, time_t *lastRef,
				    Tcl_HashSearch **search);
extern RoverErr RoverCacheSchedStat();

#endif /* HTCACHE_H */
