 /**********************************************************************
 *
 *
 *		    ROVER object execution engine
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: HTNet.h
 *
 * Description: Receives, forks, and executes a binary application
 *
 * Last Modified:  8-Feb-95 (adj)
 *
 * Global Functions:
 *    PUBLIC HTStream *HTAppRun(HTRequest *request,
 *				void *param,
 *				HTFormat input_format,
 *			        HTFormat output_format,
 *				HTStream *output_stream)
 *        Converter for application/x-application MIME stream. Downloads
 *        application binary code, creates pipes for IPC, and forks off
 *        a separate process.
 *
 * Global Variables: PUBLIC CONST HTStreamClass HTApplication
 *
 ***********************************************************************
 *   Copyright 1995
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#ifndef HTNET_H
#define HTNET_H

typedef enum _HTNetTypes {
  NONE = 0,
  LOCALHOST = 1,
  IP = 2
} HTNetTypes;

struct _HTNetIfs {
  char ifName[64];
  HTNetTypes type;
  int metric;
  char *name;
};
typedef struct _HTNetIfs HTNetIfs;

struct _HTNetState {
  int numInterfaces;
  int NetAvail;
  HTNetIfs *interfaces;
};
typedef struct _HTNetState HTNetState;

extern HTNetState RoverNetState;
extern int NetChanged;

extern int HTNetCheck();
extern void HTNetShowInterfaces();
extern void HTNetStatus();
extern char *HTNetFastInterface(int *metric, HTNetTypes  *type);


#endif /* HTNET_H */
