/**********************************************************************
 *
 *
 *		    ROVER object execution engine
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: RoverClient.h
 *
 * Description: Handles client and server side object method invocations
 *
 * Last Modified:  2-Jul-96 (adj)
 *
 ***********************************************************************
 *   Copyright 1996
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#ifndef ROVERCLIENT_H
#define ROVERCLIENT_H

#include <sys/time.h>
#include <sys/timeb.h>

#include "RoverBase.h"
#include "HTUtils.h"
#include "HTApplication.h"
#include "HTRover.h"
#include "HTSession.h"
#include "HTObject.h"
#include "HTLog.h"

struct _QRPC {
  RoverStatus status;  
  int logID;
  int count;
  char *args;
  int setArgv;
  RoverCallback callback;	/* The callback */
  char *data;
  RoverObj *obj;
};
typedef struct _QRPC QRPC;

#if ((TCL_MAJOR_VERSION > 7) || ((TCL_MAJOR_VERSION == 7) && (TCL_MINOR_VERSION> 4)))
#define NEWTCL
#define TclFD(x)	Tcl_GetFile((x), TCL_UNIX_FD)
#define TclFreeFD(x)	Tcl_FreeFile(TclFD(x))
#else
#undef NEWTCL
#define TclFD(x)	(x)
#define TclFreeFD(x)
#endif



/* Three macros to simplify things */
#define RoverCreateTclHdr(CMD) \
extern int CMD(ClientData clientData, Tcl_Interp *interp, int argc, \
		char *argv[])
#define RoverCreateTclCmd(CMD)	\
  Tcl_CreateCommand(globalTclInterp, #CMD, CMD, NULL, NULL);
#define RoverCreateTclCmdAlias(ALIAS, CMD)	\
  Tcl_CreateCommand(globalTclInterp, #ALIAS, CMD, NULL, NULL);

RoverCreateTclHdr(Rover_AddFromString);
RoverCreateTclHdr(Rover_CancelAlarm);
RoverCreateTclHdr(Rover_ClientFlush);
RoverCreateTclHdr(Rover_EndTimer);
RoverCreateTclHdr(Rover_Escape);
RoverCreateTclHdr(Rover_Export);
RoverCreateTclHdr(Rover_Flush);
RoverCreateTclHdr(Rover_GetDV);
RoverCreateTclHdr(Rover_GetPid);
RoverCreateTclHdr(Rover_Import);
RoverCreateTclHdr(Rover_Info);
RoverCreateTclHdr(Rover_IsTentative);
RoverCreateTclHdr(Rover_LoadApplication);
RoverCreateTclHdr(Rover_MarkDeleted);
RoverCreateTclHdr(Rover_NewSession);
RoverCreateTclHdr(Rover_PromiseClaim);
RoverCreateTclHdr(Rover_QRPC);
RoverCreateTclHdr(Rover_ResetSession);
RoverCreateTclHdr(Rover_RPC);
RoverCreateTclHdr(Rover_SetAlarm);
RoverCreateTclHdr(Rover_SetExportHook);
RoverCreateTclHdr(Rover_UnsetExportHook);
RoverCreateTclHdr(Rover_SetImportHook);
RoverCreateTclHdr(Rover_UnsetImportHook);
RoverCreateTclHdr(Rover_Shutdown);
RoverCreateTclHdr(Rover_StartTimer);
RoverCreateTclHdr(Rover_TclEscape);
RoverCreateTclHdr(Rover_USleep);
RoverCreateTclHdr(Rover_Unescape);
RoverCreateTclHdr(Rover_WatchFunction);

/* Defined in RoverClientTcl.h */
extern void showTclError(int rc, char *procName, Tcl_Interp *interp);
extern void Rover_UpdateLog(char *buf);
extern void Rover_UpdateApps(char *buf);
extern void Rover_UpdateHTTP(char *ptr);
extern int PromiseName(int msgID, Tcl_Interp *interp);
extern int CreatePromise(int msgID, int new, Tcl_Interp *interp);
extern int FillPromise(int msgID, int logID, char *data, Tcl_Interp *interp);
extern int FillPromiseError(int msgID, int logID, char *data,
			    Tcl_Interp *interp);
extern void Rover_QRPCCallback(int appID, int toFD, int cbID,
			       RoverType type, int size, char *buf,
			       char *data);
extern void Rover_RPCCallback(int appID, int toFD, int cbID,
			      RoverType type, int size, char *buf,
			      char *data);


PRIVATE inline char *RoverCopy(char *string)
{
  char *rv = strdup(string);
  if (!rv) fprintf(stderr, "Out of memory [RoverCopy:RoverClient.h]\n");
  return(rv);
}

PRIVATE inline char *RoverAlloc(int size)
{
  char *rv;

  if (!size) size++;
  if ((rv = (char *)malloc(size)) == NULL)
    fprintf(stderr,"Out of memory: %d [RoverAlloc:RoverClient.h]\n",size);
  return(rv);
}

PRIVATE inline char *RoverTclCopy(char *string, Tcl_Interp *interp)
{
  char *rv = strdup(string);

  if (!rv) {
    fprintf(stderr, "Out of memory [RoverTclCopy:HTRoverClient.c]\n");
    if (interp) interp->result = "Out of memory [RoverTclCopy:RoverClient.h]";
  }
  return(rv);
}

PRIVATE inline char *RoverTclAlloc(int size, Tcl_Interp *interp)
{
  char *rv;

  if (!size) size++;
  if ((rv = (char *)malloc(size)) == NULL) {
    fprintf(stderr, "Out of memory [RoverAlloc:RoverClient.h]\n");
    if (interp) interp->result = "Out of memory [RoverAlloc:RoverClient.h]";
    return(NULL);
  }
  return(rv);
}

#endif /* HTROVER_H */
